	list p=16F84A

; PIC16F84 mit 4 Mhz Takt

	#include <P16F84A.INC>
	
; Configuration festlegen:
; Power on Timer, kein Watchdog, XT-Oscillator
	
		__CONFIG	_PWRTE_ON & _WDT_OFF & _XT_OSC

; Variablen festlegen
loop		equ	0x13
temp		equ	0x14
lp4			equ	0x15
lp250		equ	0x16
rtduty		equ	0x17
gnduty		equ	0x18
blduty		equ	0x19
frun		equ 0x1A
counter		equ 0x1B



; Konstanten festlegen
#define rt		PORTB,7		;"PWM" fr Rot (amber)
#define gn		PORTB,6		;"PWM" fr Grn (true green)
#define bl		PORTB,5		;"PWM" fr Blau (blue)

; ************* Initalisierung **********************************

Init	bsf     STATUS, RP0		; auf Bank 1 umschalten
		movlw	B'00000000'		; 0 ins w, dann nach TRISB & TRISA, PORTB & PORTA alle Output
		movwf	TRISB
		movwf	TRISA
		bcf     STATUS, RP0		; auf Bank 0 zurckschalten
		movlw	.0
		movwf	PORTB			; alle Ports auf LOW
		movwf	PORTA
		clrf	loop			; alle Variablen clear
		clrf	lp4
		clrf	lp250
		clrf	rtduty
		clrf	gnduty
		clrf	blduty
		clrf	frun

; ************ Hauptprogramm ************************************

main
		movlw	.255
		movwf	frun
		movlw	.1
		movwf	rtduty
		movwf	gnduty
		movwf	blduty
runrt	call	rtloop
		bsf		rt
		incf	rtduty,f
		decfsz	frun,f
		goto	runrt
		decf	rtduty,f
		
dim		call	Wait
		decf	frun,f
addgn	call	gnloop
		bsf		gn
		incf	gnduty,f
		decfsz	frun,f
		goto	addgn
		decf	gnduty,f
		
		call	Wait
		decf	frun,f
subrt	call	rtloop
		decf	rtduty,f
		decfsz	frun,f
		goto	subrt
		incf	rtduty,f
		
		call	Wait
		decf	frun,f
addbl	call	blloop
		bsf		bl
		incf	blduty,f
		decfsz	frun,f
		goto	addbl
		decf	blduty,f
		
		call	Wait
		decf	frun,f
subgn	call	gnloop
		decf	gnduty,f
		decfsz	frun,f
		goto	subgn
		incf	gnduty,f
		
		call	Wait
		decf	frun,f
addrt	call	rtloop
		bsf		rt
		incf	rtduty,f
		decfsz	frun,f
		goto	addrt
		decf	rtduty,f
		
		call	Wait
		decf	frun,f
subbl	call	blloop
		decf	blduty,f
		decfsz	frun,f
		goto	subbl
		incf	blduty,f

		goto	dim

; ************ PWM-Rot generieren ****************

rtloop
		movlw .30		;Zhler fr die bergangsgeschwindigkeit
		movwf counter
count_rt
		movf rtduty,w
		movwf temp
		bsf rt
pwmrt1	nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmrt1
		movf rtduty,w
		subwf temp,f
		bcf rt
pwmrt2
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmrt2
		decfsz counter,f
		goto count_rt
		return

; ************ PWM-Grn generieren ****************

gnloop
		movlw .30		;Zhler fr die bergangsgeschwindigkeit
		movwf counter
count_gn
		movf gnduty,w
		movwf temp
		bsf gn
pwmgn1	nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmgn1
		movf gnduty,w
		subwf temp,f
		bcf gn
pwmgn2
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmgn2
		decfsz counter,f
		goto count_gn
		return

; ************ PWM-Blau generieren ****************

blloop
		movlw .30		;Zhler fr die bergangsgeschwindigkeit
		movwf counter
count_bl
		movf blduty,w
		movwf temp
		bsf bl
pwmbl1	nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmbl1
		movf blduty,w
		subwf temp,f
		bcf bl
pwmbl2
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz temp,f
		goto pwmbl2
		decfsz counter,f
		goto count_bl
		return

; ************ Warteschleifen ***********************************

; 1 Sekunde warten
Wait
		movlw	.4
		movwf	lp4
w1sek
		
; 250ms warten
		movlw	.250
		movwf	lp250
w250
		call	ms1
		decfsz	lp250,f		;250ms vorbei?
		goto	w250

		decfsz	lp4,f		;1 Sek vorbei?
		goto	w1sek
		return				;Ja

; Zeitkonstante fr 1ms
ms1
		movlw	.110
		movwf	loop	
Wait2
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz	loop, F		;1ms vorbei?
		goto	Wait2		;nein, noch nicht
		return				;Ende mit 1ms
		
; ENDE der Warteschleifen ***************************************


		end
